# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import class_list_params, class_create_params, class_update_params
from ..._base_client import make_request_options
from ...types.qbd.class_ import Class
from ...types.qbd.class_list_response import ClassListResponse

__all__ = ["ClassesResource", "AsyncClassesResource"]


class ClassesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ClassesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return ClassesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClassesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return ClassesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        conductor_end_user_id: str,
        is_active: bool | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Class:
        """Creates a new class.

        Args:
          name: The case-insensitive name of this class.

        Not guaranteed to be unique because it
              does not include the names of its hierarchical parent objects like `fullName`
              does. For example, two classes could both have the `name` "Marketing", but they
              could have unique `fullName` values, such as "Department:Marketing" and
              "Internal:Marketing".

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          is_active: Indicates whether this class is active. Inactive objects are typically hidden
              from views and reports in QuickBooks. Defaults to `true`.

          parent_id: The parent class one level above this one in the hierarchy. For example, if this
              class has a `fullName` of "Department:Marketing", its parent has a `fullName` of
              "Department". If this class is at the top level, this field will be `null`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/classes",
            body=maybe_transform(
                {
                    "name": name,
                    "is_active": is_active,
                    "parent_id": parent_id,
                },
                class_create_params.ClassCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Class,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Class:
        """
        Retrieves a class by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the class to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/classes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Class,
        )

    def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        is_active: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Class:
        """
        Updates an existing class.

        Args:
          id: The QuickBooks-assigned unique identifier of the class to update.

          revision_number: The current QuickBooks-assigned revision number of the class object you are
              updating, which you can get by fetching the object first. Provide the most
              recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          is_active: Indicates whether this class is active. Inactive objects are typically hidden
              from views and reports in QuickBooks. Defaults to `true`.

          name: The case-insensitive name of this class. Not guaranteed to be unique because it
              does not include the names of its hierarchical parent objects like `fullName`
              does. For example, two classes could both have the `name` "Marketing", but they
              could have unique `fullName` values, such as "Department:Marketing" and
              "Internal:Marketing".

              Maximum length: 31 characters.

          parent_id: The parent class one level above this one in the hierarchy. For example, if this
              class has a `fullName` of "Department:Marketing", its parent has a `fullName` of
              "Department". If this class is at the top level, this field will be `null`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            f"/quickbooks-desktop/classes/{id}",
            body=maybe_transform(
                {
                    "revision_number": revision_number,
                    "is_active": is_active,
                    "name": name,
                    "parent_id": parent_id,
                },
                class_update_params.ClassUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Class,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        full_names: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClassListResponse:
        """Returns a list of classes.

        NOTE: QuickBooks Desktop does not support pagination
        for classes; hence, there is no `cursor` parameter. Users typically have few
        classes.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          full_names: Filter for specific classes by their full-name(s), case-insensitive. Like `id`,
              `fullName` is a unique identifier for a class, formed by by combining the names
              of its parent objects with its own `name`, separated by colons. For example, if
              a class is under "Department" and has the `name` "Marketing", its `fullName`
              would be "Department:Marketing".

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ids: Filter for specific classes by their QuickBooks-assigned unique identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return.

              **IMPORTANT**: QuickBooks Desktop does not support cursor-based pagination for
              classes. This parameter will limit the response size, but you cannot fetch
              subsequent results using a cursor. For pagination, use the name-range parameters
              instead (e.g., `nameFrom=A&nameTo=B`).

              When this parameter is omitted, the endpoint returns all classes without limit,
              unlike paginated endpoints which default to 150 records. This is acceptable
              because classes typically have low record counts.

          name_contains:
              Filter for classes whose `name` contains this substring, case-insensitive. NOTE:
              If you use this parameter, you cannot also use `nameStartsWith` or
              `nameEndsWith`.

          name_ends_with: Filter for classes whose `name` ends with this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameContains` or
              `nameStartsWith`.

          name_from: Filter for classes whose `name` is alphabetically greater than or equal to this
              value.

          name_starts_with: Filter for classes whose `name` starts with this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameContains` or
              `nameEndsWith`.

          name_to: Filter for classes whose `name` is alphabetically less than or equal to this
              value.

          status: Filter for classes that are active, inactive, or both.

          updated_after: Filter for classes updated on or after this date and time, in ISO 8601 format
              (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
              assumed to be 00:00:00 of that day.

          updated_before: Filter for classes updated on or before this date and time, in ISO 8601 format
              (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
              assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            "/quickbooks-desktop/classes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "full_names": full_names,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    class_list_params.ClassListParams,
                ),
            ),
            cast_to=ClassListResponse,
        )


class AsyncClassesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncClassesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncClassesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClassesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncClassesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        conductor_end_user_id: str,
        is_active: bool | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Class:
        """Creates a new class.

        Args:
          name: The case-insensitive name of this class.

        Not guaranteed to be unique because it
              does not include the names of its hierarchical parent objects like `fullName`
              does. For example, two classes could both have the `name` "Marketing", but they
              could have unique `fullName` values, such as "Department:Marketing" and
              "Internal:Marketing".

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          is_active: Indicates whether this class is active. Inactive objects are typically hidden
              from views and reports in QuickBooks. Defaults to `true`.

          parent_id: The parent class one level above this one in the hierarchy. For example, if this
              class has a `fullName` of "Department:Marketing", its parent has a `fullName` of
              "Department". If this class is at the top level, this field will be `null`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/classes",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "is_active": is_active,
                    "parent_id": parent_id,
                },
                class_create_params.ClassCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Class,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Class:
        """
        Retrieves a class by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the class to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/classes/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Class,
        )

    async def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        is_active: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Class:
        """
        Updates an existing class.

        Args:
          id: The QuickBooks-assigned unique identifier of the class to update.

          revision_number: The current QuickBooks-assigned revision number of the class object you are
              updating, which you can get by fetching the object first. Provide the most
              recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          is_active: Indicates whether this class is active. Inactive objects are typically hidden
              from views and reports in QuickBooks. Defaults to `true`.

          name: The case-insensitive name of this class. Not guaranteed to be unique because it
              does not include the names of its hierarchical parent objects like `fullName`
              does. For example, two classes could both have the `name` "Marketing", but they
              could have unique `fullName` values, such as "Department:Marketing" and
              "Internal:Marketing".

              Maximum length: 31 characters.

          parent_id: The parent class one level above this one in the hierarchy. For example, if this
              class has a `fullName` of "Department:Marketing", its parent has a `fullName` of
              "Department". If this class is at the top level, this field will be `null`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            f"/quickbooks-desktop/classes/{id}",
            body=await async_maybe_transform(
                {
                    "revision_number": revision_number,
                    "is_active": is_active,
                    "name": name,
                    "parent_id": parent_id,
                },
                class_update_params.ClassUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Class,
        )

    async def list(
        self,
        *,
        conductor_end_user_id: str,
        full_names: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClassListResponse:
        """Returns a list of classes.

        NOTE: QuickBooks Desktop does not support pagination
        for classes; hence, there is no `cursor` parameter. Users typically have few
        classes.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          full_names: Filter for specific classes by their full-name(s), case-insensitive. Like `id`,
              `fullName` is a unique identifier for a class, formed by by combining the names
              of its parent objects with its own `name`, separated by colons. For example, if
              a class is under "Department" and has the `name` "Marketing", its `fullName`
              would be "Department:Marketing".

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ids: Filter for specific classes by their QuickBooks-assigned unique identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return.

              **IMPORTANT**: QuickBooks Desktop does not support cursor-based pagination for
              classes. This parameter will limit the response size, but you cannot fetch
              subsequent results using a cursor. For pagination, use the name-range parameters
              instead (e.g., `nameFrom=A&nameTo=B`).

              When this parameter is omitted, the endpoint returns all classes without limit,
              unlike paginated endpoints which default to 150 records. This is acceptable
              because classes typically have low record counts.

          name_contains:
              Filter for classes whose `name` contains this substring, case-insensitive. NOTE:
              If you use this parameter, you cannot also use `nameStartsWith` or
              `nameEndsWith`.

          name_ends_with: Filter for classes whose `name` ends with this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameContains` or
              `nameStartsWith`.

          name_from: Filter for classes whose `name` is alphabetically greater than or equal to this
              value.

          name_starts_with: Filter for classes whose `name` starts with this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameContains` or
              `nameEndsWith`.

          name_to: Filter for classes whose `name` is alphabetically less than or equal to this
              value.

          status: Filter for classes that are active, inactive, or both.

          updated_after: Filter for classes updated on or after this date and time, in ISO 8601 format
              (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
              assumed to be 00:00:00 of that day.

          updated_before: Filter for classes updated on or before this date and time, in ISO 8601 format
              (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
              assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            "/quickbooks-desktop/classes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "full_names": full_names,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    class_list_params.ClassListParams,
                ),
            ),
            cast_to=ClassListResponse,
        )


class ClassesResourceWithRawResponse:
    def __init__(self, classes: ClassesResource) -> None:
        self._classes = classes

        self.create = to_raw_response_wrapper(
            classes.create,
        )
        self.retrieve = to_raw_response_wrapper(
            classes.retrieve,
        )
        self.update = to_raw_response_wrapper(
            classes.update,
        )
        self.list = to_raw_response_wrapper(
            classes.list,
        )


class AsyncClassesResourceWithRawResponse:
    def __init__(self, classes: AsyncClassesResource) -> None:
        self._classes = classes

        self.create = async_to_raw_response_wrapper(
            classes.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            classes.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            classes.update,
        )
        self.list = async_to_raw_response_wrapper(
            classes.list,
        )


class ClassesResourceWithStreamingResponse:
    def __init__(self, classes: ClassesResource) -> None:
        self._classes = classes

        self.create = to_streamed_response_wrapper(
            classes.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            classes.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            classes.update,
        )
        self.list = to_streamed_response_wrapper(
            classes.list,
        )


class AsyncClassesResourceWithStreamingResponse:
    def __init__(self, classes: AsyncClassesResource) -> None:
        self._classes = classes

        self.create = async_to_streamed_response_wrapper(
            classes.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            classes.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            classes.update,
        )
        self.list = async_to_streamed_response_wrapper(
            classes.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import bill_credit_card_payment_list_params, bill_credit_card_payment_create_params
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.qbd.bill_credit_card_payment import BillCreditCardPayment
from ...types.qbd.bill_credit_card_payment_delete_response import BillCreditCardPaymentDeleteResponse

__all__ = ["BillCreditCardPaymentsResource", "AsyncBillCreditCardPaymentsResource"]


class BillCreditCardPaymentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BillCreditCardPaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return BillCreditCardPaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BillCreditCardPaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return BillCreditCardPaymentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        apply_to_transactions: Iterable[bill_credit_card_payment_create_params.ApplyToTransaction],
        credit_card_account_id: str,
        transaction_date: Union[str, date],
        vendor_id: str,
        conductor_end_user_id: str,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payables_account_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillCreditCardPayment:
        """
        Creates a new bill credit card payment.

        Args:
          apply_to_transactions: The bills to be paid by this bill credit card payment. This will create a link
              between this bill credit card payment and the specified bills.

              **IMPORTANT**: In each `applyToTransactions` object, you must specify either
              `paymentAmount`, `applyCredits`, `discountAmount`, or any combination of these;
              if none of these are specified, you will receive an error for an empty
              transaction.

              **IMPORTANT**: The target bill must have `isPaid=false`, otherwise, QuickBooks
              will report this object as "cannot be found".

          credit_card_account_id: The credit card account to which this bill credit card payment is being charged.
              This bill credit card payment will decrease the balance of this account.

          transaction_date: The date of this bill credit card payment, in ISO 8601 format (YYYY-MM-DD).

          vendor_id: The vendor who sent the bill(s) that this bill credit card payment is paying and
              who will receive this payment.

              **IMPORTANT**: This vendor must match the `vendor` on the bill(s) specified in
              `applyToTransactions`; otherwise, QuickBooks will say the `transactionId` in
              `applyToTransactions` "does not exist".

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          exchange_rate: The market exchange rate between this bill credit card payment's currency and
              the home currency in QuickBooks at the time of this transaction. Represented as
              a decimal value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home
              currency).

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          memo: A memo or note for this bill credit card payment.

          payables_account_id: The Accounts-Payable (A/P) account to which this bill credit card payment is
              assigned, used to track the amount owed. If not specified, QuickBooks Desktop
              will use its default A/P account.

              **IMPORTANT**: If this bill credit card payment is linked to other transactions,
              this A/P account must match the `payablesAccount` used in those other
              transactions.

          ref_number: The case-sensitive user-defined reference number for this bill credit card
              payment, which can be used to identify the transaction in QuickBooks. This value
              is not required to be unique and can be arbitrarily changed by the QuickBooks
              user. When left blank in this create request, this field will be left blank in
              QuickBooks (i.e., it does _not_ auto-increment).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/bill-credit-card-payments",
            body=maybe_transform(
                {
                    "apply_to_transactions": apply_to_transactions,
                    "credit_card_account_id": credit_card_account_id,
                    "transaction_date": transaction_date,
                    "vendor_id": vendor_id,
                    "exchange_rate": exchange_rate,
                    "external_id": external_id,
                    "memo": memo,
                    "payables_account_id": payables_account_id,
                    "ref_number": ref_number,
                },
                bill_credit_card_payment_create_params.BillCreditCardPaymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillCreditCardPayment,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillCreditCardPayment:
        """
        Retrieves a bill credit card payment by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the bill credit card payment to
              retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/bill-credit-card-payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillCreditCardPayment,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        currency_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_line_items: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        ref_number_contains: str | NotGiven = NOT_GIVEN,
        ref_number_ends_with: str | NotGiven = NOT_GIVEN,
        ref_number_from: str | NotGiven = NOT_GIVEN,
        ref_numbers: List[str] | NotGiven = NOT_GIVEN,
        ref_number_starts_with: str | NotGiven = NOT_GIVEN,
        ref_number_to: str | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        vendor_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[BillCreditCardPayment]:
        """Returns a list of bill credit card payments.

        Use the `cursor` parameter to
        paginate through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_ids: Filter for bill credit card payments associated with these accounts.

          currency_ids: Filter for bill credit card payments in these currencies.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          ids: Filter for specific bill credit card payments by their QuickBooks-assigned
              unique identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          include_line_items: Whether to include line items in the response. Defaults to `true`.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          ref_number_contains: Filter for bill credit card payments whose `refNumber` contains this substring.
              NOTE: If you use this parameter, you cannot also use `refNumberStartsWith` or
              `refNumberEndsWith`.

          ref_number_ends_with: Filter for bill credit card payments whose `refNumber` ends with this substring.
              NOTE: If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberStartsWith`.

          ref_number_from: Filter for bill credit card payments whose `refNumber` is greater than or equal
              to this value. If omitted, the range will begin with the first number of the
              list. Uses a numerical comparison for values that contain only digits;
              otherwise, uses a lexicographical comparison.

          ref_numbers: Filter for specific bill credit card payments by their ref-number(s),
              case-sensitive. In QuickBooks, ref-numbers are not required to be unique and can
              be arbitrarily changed by the QuickBooks user.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ref_number_starts_with: Filter for bill credit card payments whose `refNumber` starts with this
              substring. NOTE: If you use this parameter, you cannot also use
              `refNumberContains` or `refNumberEndsWith`.

          ref_number_to: Filter for bill credit card payments whose `refNumber` is less than or equal to
              this value. If omitted, the range will end with the last number of the list.
              Uses a numerical comparison for values that contain only digits; otherwise, uses
              a lexicographical comparison.

          transaction_date_from: Filter for bill credit card payments whose `date` field is on or after this
              date, in ISO 8601 format (YYYY-MM-DD).

          transaction_date_to: Filter for bill credit card payments whose `date` field is on or before this
              date, in ISO 8601 format (YYYY-MM-DD).

          updated_after: Filter for bill credit card payments updated on or after this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 00:00:00 of that day.

          updated_before: Filter for bill credit card payments updated on or before this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 23:59:59 of that day.

          vendor_ids: Filter for bill credit card payments sent to these vendors. These are the
              vendors who sent the bills paid by these credit card payments.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/bill-credit-card-payments",
            page=SyncCursorPage[BillCreditCardPayment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_ids": account_ids,
                        "currency_ids": currency_ids,
                        "cursor": cursor,
                        "ids": ids,
                        "include_line_items": include_line_items,
                        "limit": limit,
                        "ref_number_contains": ref_number_contains,
                        "ref_number_ends_with": ref_number_ends_with,
                        "ref_number_from": ref_number_from,
                        "ref_numbers": ref_numbers,
                        "ref_number_starts_with": ref_number_starts_with,
                        "ref_number_to": ref_number_to,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                        "vendor_ids": vendor_ids,
                    },
                    bill_credit_card_payment_list_params.BillCreditCardPaymentListParams,
                ),
            ),
            model=BillCreditCardPayment,
        )

    def delete(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillCreditCardPaymentDeleteResponse:
        """Permanently deletes a a bill credit card payment.

        The deletion will fail if the
        bill credit card payment is currently in use or has any linked transactions that
        are in use.

        Args:
          id: The QuickBooks-assigned unique identifier of the bill credit card payment to
              delete.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._delete(
            f"/quickbooks-desktop/bill-credit-card-payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillCreditCardPaymentDeleteResponse,
        )


class AsyncBillCreditCardPaymentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBillCreditCardPaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBillCreditCardPaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBillCreditCardPaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncBillCreditCardPaymentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        apply_to_transactions: Iterable[bill_credit_card_payment_create_params.ApplyToTransaction],
        credit_card_account_id: str,
        transaction_date: Union[str, date],
        vendor_id: str,
        conductor_end_user_id: str,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payables_account_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillCreditCardPayment:
        """
        Creates a new bill credit card payment.

        Args:
          apply_to_transactions: The bills to be paid by this bill credit card payment. This will create a link
              between this bill credit card payment and the specified bills.

              **IMPORTANT**: In each `applyToTransactions` object, you must specify either
              `paymentAmount`, `applyCredits`, `discountAmount`, or any combination of these;
              if none of these are specified, you will receive an error for an empty
              transaction.

              **IMPORTANT**: The target bill must have `isPaid=false`, otherwise, QuickBooks
              will report this object as "cannot be found".

          credit_card_account_id: The credit card account to which this bill credit card payment is being charged.
              This bill credit card payment will decrease the balance of this account.

          transaction_date: The date of this bill credit card payment, in ISO 8601 format (YYYY-MM-DD).

          vendor_id: The vendor who sent the bill(s) that this bill credit card payment is paying and
              who will receive this payment.

              **IMPORTANT**: This vendor must match the `vendor` on the bill(s) specified in
              `applyToTransactions`; otherwise, QuickBooks will say the `transactionId` in
              `applyToTransactions` "does not exist".

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          exchange_rate: The market exchange rate between this bill credit card payment's currency and
              the home currency in QuickBooks at the time of this transaction. Represented as
              a decimal value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home
              currency).

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          memo: A memo or note for this bill credit card payment.

          payables_account_id: The Accounts-Payable (A/P) account to which this bill credit card payment is
              assigned, used to track the amount owed. If not specified, QuickBooks Desktop
              will use its default A/P account.

              **IMPORTANT**: If this bill credit card payment is linked to other transactions,
              this A/P account must match the `payablesAccount` used in those other
              transactions.

          ref_number: The case-sensitive user-defined reference number for this bill credit card
              payment, which can be used to identify the transaction in QuickBooks. This value
              is not required to be unique and can be arbitrarily changed by the QuickBooks
              user. When left blank in this create request, this field will be left blank in
              QuickBooks (i.e., it does _not_ auto-increment).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/bill-credit-card-payments",
            body=await async_maybe_transform(
                {
                    "apply_to_transactions": apply_to_transactions,
                    "credit_card_account_id": credit_card_account_id,
                    "transaction_date": transaction_date,
                    "vendor_id": vendor_id,
                    "exchange_rate": exchange_rate,
                    "external_id": external_id,
                    "memo": memo,
                    "payables_account_id": payables_account_id,
                    "ref_number": ref_number,
                },
                bill_credit_card_payment_create_params.BillCreditCardPaymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillCreditCardPayment,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillCreditCardPayment:
        """
        Retrieves a bill credit card payment by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the bill credit card payment to
              retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/bill-credit-card-payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillCreditCardPayment,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        currency_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_line_items: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        ref_number_contains: str | NotGiven = NOT_GIVEN,
        ref_number_ends_with: str | NotGiven = NOT_GIVEN,
        ref_number_from: str | NotGiven = NOT_GIVEN,
        ref_numbers: List[str] | NotGiven = NOT_GIVEN,
        ref_number_starts_with: str | NotGiven = NOT_GIVEN,
        ref_number_to: str | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        vendor_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[BillCreditCardPayment, AsyncCursorPage[BillCreditCardPayment]]:
        """Returns a list of bill credit card payments.

        Use the `cursor` parameter to
        paginate through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_ids: Filter for bill credit card payments associated with these accounts.

          currency_ids: Filter for bill credit card payments in these currencies.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          ids: Filter for specific bill credit card payments by their QuickBooks-assigned
              unique identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          include_line_items: Whether to include line items in the response. Defaults to `true`.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          ref_number_contains: Filter for bill credit card payments whose `refNumber` contains this substring.
              NOTE: If you use this parameter, you cannot also use `refNumberStartsWith` or
              `refNumberEndsWith`.

          ref_number_ends_with: Filter for bill credit card payments whose `refNumber` ends with this substring.
              NOTE: If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberStartsWith`.

          ref_number_from: Filter for bill credit card payments whose `refNumber` is greater than or equal
              to this value. If omitted, the range will begin with the first number of the
              list. Uses a numerical comparison for values that contain only digits;
              otherwise, uses a lexicographical comparison.

          ref_numbers: Filter for specific bill credit card payments by their ref-number(s),
              case-sensitive. In QuickBooks, ref-numbers are not required to be unique and can
              be arbitrarily changed by the QuickBooks user.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ref_number_starts_with: Filter for bill credit card payments whose `refNumber` starts with this
              substring. NOTE: If you use this parameter, you cannot also use
              `refNumberContains` or `refNumberEndsWith`.

          ref_number_to: Filter for bill credit card payments whose `refNumber` is less than or equal to
              this value. If omitted, the range will end with the last number of the list.
              Uses a numerical comparison for values that contain only digits; otherwise, uses
              a lexicographical comparison.

          transaction_date_from: Filter for bill credit card payments whose `date` field is on or after this
              date, in ISO 8601 format (YYYY-MM-DD).

          transaction_date_to: Filter for bill credit card payments whose `date` field is on or before this
              date, in ISO 8601 format (YYYY-MM-DD).

          updated_after: Filter for bill credit card payments updated on or after this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 00:00:00 of that day.

          updated_before: Filter for bill credit card payments updated on or before this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 23:59:59 of that day.

          vendor_ids: Filter for bill credit card payments sent to these vendors. These are the
              vendors who sent the bills paid by these credit card payments.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/bill-credit-card-payments",
            page=AsyncCursorPage[BillCreditCardPayment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_ids": account_ids,
                        "currency_ids": currency_ids,
                        "cursor": cursor,
                        "ids": ids,
                        "include_line_items": include_line_items,
                        "limit": limit,
                        "ref_number_contains": ref_number_contains,
                        "ref_number_ends_with": ref_number_ends_with,
                        "ref_number_from": ref_number_from,
                        "ref_numbers": ref_numbers,
                        "ref_number_starts_with": ref_number_starts_with,
                        "ref_number_to": ref_number_to,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                        "vendor_ids": vendor_ids,
                    },
                    bill_credit_card_payment_list_params.BillCreditCardPaymentListParams,
                ),
            ),
            model=BillCreditCardPayment,
        )

    async def delete(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillCreditCardPaymentDeleteResponse:
        """Permanently deletes a a bill credit card payment.

        The deletion will fail if the
        bill credit card payment is currently in use or has any linked transactions that
        are in use.

        Args:
          id: The QuickBooks-assigned unique identifier of the bill credit card payment to
              delete.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._delete(
            f"/quickbooks-desktop/bill-credit-card-payments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillCreditCardPaymentDeleteResponse,
        )


class BillCreditCardPaymentsResourceWithRawResponse:
    def __init__(self, bill_credit_card_payments: BillCreditCardPaymentsResource) -> None:
        self._bill_credit_card_payments = bill_credit_card_payments

        self.create = to_raw_response_wrapper(
            bill_credit_card_payments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            bill_credit_card_payments.retrieve,
        )
        self.list = to_raw_response_wrapper(
            bill_credit_card_payments.list,
        )
        self.delete = to_raw_response_wrapper(
            bill_credit_card_payments.delete,
        )


class AsyncBillCreditCardPaymentsResourceWithRawResponse:
    def __init__(self, bill_credit_card_payments: AsyncBillCreditCardPaymentsResource) -> None:
        self._bill_credit_card_payments = bill_credit_card_payments

        self.create = async_to_raw_response_wrapper(
            bill_credit_card_payments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            bill_credit_card_payments.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            bill_credit_card_payments.list,
        )
        self.delete = async_to_raw_response_wrapper(
            bill_credit_card_payments.delete,
        )


class BillCreditCardPaymentsResourceWithStreamingResponse:
    def __init__(self, bill_credit_card_payments: BillCreditCardPaymentsResource) -> None:
        self._bill_credit_card_payments = bill_credit_card_payments

        self.create = to_streamed_response_wrapper(
            bill_credit_card_payments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            bill_credit_card_payments.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            bill_credit_card_payments.list,
        )
        self.delete = to_streamed_response_wrapper(
            bill_credit_card_payments.delete,
        )


class AsyncBillCreditCardPaymentsResourceWithStreamingResponse:
    def __init__(self, bill_credit_card_payments: AsyncBillCreditCardPaymentsResource) -> None:
        self._bill_credit_card_payments = bill_credit_card_payments

        self.create = async_to_streamed_response_wrapper(
            bill_credit_card_payments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            bill_credit_card_payments.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            bill_credit_card_payments.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            bill_credit_card_payments.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CreditMemoDeleteResponse"]


class CreditMemoDeleteResponse(BaseModel):
    id: str
    """The QuickBooks-assigned unique identifier of the deleted credit memo."""

    deleted: bool
    """Indicates whether the credit memo was deleted."""

    object_type: Literal["qbd_credit_memo"] = FieldInfo(alias="objectType")
    """The type of object. This value is always `"qbd_credit_memo"`."""

    ref_number: Optional[str] = FieldInfo(alias="refNumber", default=None)
    """The case-sensitive user-defined reference number of the deleted credit memo."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .sales_representative import SalesRepresentative

__all__ = ["SalesRepresentativeListResponse"]


class SalesRepresentativeListResponse(BaseModel):
    data: List[SalesRepresentative]
    """The array of sales representatives."""

    object_type: Literal["list"] = FieldInfo(alias="objectType")
    """The type of object. This value is always `"list"`."""

    url: str
    """The endpoint URL where this list can be accessed."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["AccountTaxLine"]


class AccountTaxLine(BaseModel):
    tax_line_id: float = FieldInfo(alias="taxLineId")
    """The identifier of the tax line associated with this account tax line.

    You can see a list of all available values for this field by calling the
    endpoint for account tax lines.
    """

    tax_line_name: Optional[str] = FieldInfo(alias="taxLineName", default=None)
    """
    The name of the tax line associated with this account tax line, as it appears on
    the tax form.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import date
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "Preferences",
    "Accounting",
    "AppAccessRights",
    "FinanceCharges",
    "FinanceChargesFinanceChargeAccount",
    "ItemsAndInventory",
    "JobsAndEstimates",
    "MultiCurrency",
    "MultiCurrencyHomeCurrency",
    "MultiLocationInventory",
    "PurchasesAndVendors",
    "PurchasesAndVendorsDefaultDiscountAccount",
    "Reports",
    "SalesAndCustomers",
    "SalesAndCustomersDefaultShippingMethod",
    "SalesAndCustomersPriceLevels",
    "SalesTax",
    "SalesTaxDefaultItemSalesTax",
    "SalesTaxDefaultNonTaxableSalesTaxCode",
    "SalesTaxDefaultTaxableSalesTaxCode",
    "TimeTracking",
]


class Accounting(BaseModel):
    closing_date: Optional[date] = FieldInfo(alias="closingDate", default=None)
    """The company closing date set within this company file.

    (The QuickBooks Admin can assign a password restricting access to transactions
    that occurred before this date.)
    """

    default_transaction_class: Optional[Literal["accounts", "items", "names", "none"]] = FieldInfo(
        alias="defaultTransactionClass", default=None
    )
    """The default class assigned to transactions for this company file."""

    is_assigning_journal_entry_numbers: bool = FieldInfo(alias="isAssigningJournalEntryNumbers")
    """
    Indicates whether this company file is configured to automatically assign a
    number to each journal entry.
    """

    is_requiring_accounts: bool = FieldInfo(alias="isRequiringAccounts")
    """
    Indicates whether this company file is configured to require an account for new
    transactions. If `true`, a transaction cannot be recorded in the QuickBooks user
    interface unless it is assigned to an account. (However, transactions affected
    by this preference always require an account to be specified when added through
    the API.)
    """

    is_using_account_numbers: bool = FieldInfo(alias="isUsingAccountNumbers")
    """
    Indicates whether this company file is configured to record an account number
    for new accounts. If you include an account number when creating a new account
    while this preference is `false`, the account number will still be set, but will
    not be visible in the QuickBooks user interface.
    """

    is_using_audit_trail: bool = FieldInfo(alias="isUsingAuditTrail")
    """
    Indicates whether this company file is configured to log all transaction changes
    in the audit trail report. If `false`, QuickBooks logs only the most recent
    version of each transaction.
    """

    is_using_class_tracking: bool = FieldInfo(alias="isUsingClassTracking")
    """
    Indicates whether this company file is configured to use the `class` field on
    all transactions.
    """


class AppAccessRights(BaseModel):
    automatic_login_user_name: Optional[str] = FieldInfo(alias="automaticLoginUserName", default=None)
    """
    If auto-login is allowed for this company file, specifies the user name that is
    allowed to use auto-login.
    """

    is_automatic_login_allowed: bool = FieldInfo(alias="isAutomaticLoginAllowed")
    """Indicates whether applications can use auto-login to access this company file."""

    is_personal_data_access_allowed: bool = FieldInfo(alias="isPersonalDataAccessAllowed")
    """
    Indicates whether access is allowed to personal (sensitive) data in this company
    file.
    """


class FinanceChargesFinanceChargeAccount(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class FinanceCharges(BaseModel):
    annual_interest_rate: Optional[float] = FieldInfo(alias="annualInterestRate", default=None)
    """
    The interest rate that QuickBooks will use to calculate finance charges for this
    company file. Default is `0`.
    """

    calculate_charges_from: Literal["due_date", "invoice_or_billed_date"] = FieldInfo(alias="calculateChargesFrom")
    """The date from which finance charges are calculated for this company file.

    Default is `due_date`.
    """

    finance_charge_account: Optional[FinanceChargesFinanceChargeAccount] = FieldInfo(
        alias="financeChargeAccount", default=None
    )
    """
    The account used to track finance charges that customers pay for this company
    file. This is usually an income account.
    """

    grace_period: float = FieldInfo(alias="gracePeriod")
    """
    The number of days before finance charges apply to customers' overdue invoices
    for this company file. Default is `0`.
    """

    is_assessing_for_overdue_charges: bool = FieldInfo(alias="isAssessingForOverdueCharges")
    """
    Indicates whether this company file is configured to assess finance charges for
    overdue invoices. Default is `false`. (Note that laws vary about whether a
    company can charge interest on overdue interest payments.)
    """

    is_marked_to_be_printed: bool = FieldInfo(alias="isMarkedToBePrinted")
    """
    Indicates whether this company file is configured to mark all newly created
    finance-charge invoices as "to be printed". Default is `false`. The user can
    still change this preference for each individual invoice.
    """

    minimum_finance_charge: Optional[float] = FieldInfo(alias="minimumFinanceCharge", default=None)
    """
    The minimum finance charge that will be applied regardless of the amount overdue
    for this company file. Default is `0`.
    """


class ItemsAndInventory(BaseModel):
    fifo_effective_date: Optional[date] = FieldInfo(alias="fifoEffectiveDate", default=None)
    """
    The date from which FIFO (First In, First Out) is used to calculate the value of
    inventory sold and on-hand for this company file, in ISO 8601 format
    (YYYY-MM-DD).
    """

    inventory_tracking_method: Optional[Literal["none", "serial_number", "lot_number"]] = FieldInfo(
        alias="inventoryTrackingMethod", default=None
    )
    """Specifies the type of inventory tracking that this company file uses."""

    is_barcode_enabled: Optional[bool] = FieldInfo(alias="isBarcodeEnabled", default=None)
    """Indicates whether barcode functionality is enabled for this company file."""

    is_bin_tracking_enabled: Optional[bool] = FieldInfo(alias="isBinTrackingEnabled", default=None)
    """Indicates whether bin tracking is enabled for this company file.

    When `true`, inventory can be tracked by bin locations within sites.
    """

    is_enhanced_inventory_receiving_enabled: Optional[bool] = FieldInfo(
        alias="isEnhancedInventoryReceivingEnabled", default=None
    )
    """
    Indicates whether enhanced inventory receiving is enabled for this company file.
    """

    is_fifo_enabled: Optional[bool] = FieldInfo(alias="isFifoEnabled", default=None)
    """
    Indicates whether this company file is configured to use FIFO (First In, First
    Out) to calculate the value of inventory sold and on-hand.
    """

    is_inventory_expiration_date_enabled: Optional[bool] = FieldInfo(
        alias="isInventoryExpirationDateEnabled", default=None
    )
    """
    Indicates whether expiration dates for inventory serial/lot numbers are enabled
    for this company file. This feature is supported from QuickBooks Desktop 2023.
    """

    is_tracking_on_build_assembly_enabled: Optional[bool] = FieldInfo(
        alias="isTrackingOnBuildAssemblyEnabled", default=None
    )
    """
    Indicates whether serial/lot number tracking is enabled for build assemblies in
    this company file.
    """

    is_tracking_on_inventory_adjustment_enabled: Optional[bool] = FieldInfo(
        alias="isTrackingOnInventoryAdjustmentEnabled", default=None
    )
    """
    Indicates whether serial/lot number tracking is enabled for inventory
    adjustments in this company file.
    """

    is_tracking_on_purchase_transactions_enabled: Optional[bool] = FieldInfo(
        alias="isTrackingOnPurchaseTransactionsEnabled", default=None
    )
    """
    Indicates whether serial/lot number tracking is enabled for purchase
    transactions in this company file.
    """

    is_tracking_on_sales_transactions_enabled: Optional[bool] = FieldInfo(
        alias="isTrackingOnSalesTransactionsEnabled", default=None
    )
    """
    Indicates whether serial/lot number tracking is enabled for sales transactions
    in this company file.
    """


class JobsAndEstimates(BaseModel):
    is_printing_items_with_zero_amounts: bool = FieldInfo(alias="isPrintingItemsWithZeroAmounts")
    """
    Indicates whether this company file is configured to print line items with zero
    amounts on progress invoices. This preference is only relevant if
    `isUsingProgressInvoicing` is `true`.
    """

    is_using_estimates: bool = FieldInfo(alias="isUsingEstimates")
    """Indicates whether this company file is configured to create estimates for jobs."""

    is_using_progress_invoicing: bool = FieldInfo(alias="isUsingProgressInvoicing")
    """
    Indicates whether this company file permits creating invoices for only a portion
    of an estimate.
    """


class MultiCurrencyHomeCurrency(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class MultiCurrency(BaseModel):
    home_currency: Optional[MultiCurrencyHomeCurrency] = FieldInfo(alias="homeCurrency", default=None)
    """The currency that is set as the home currency for this company file.

    The home currency is normally the currency of the country where the business is
    physically located. Although a home currency other than US Dollars can be
    chosen, certain QuickBooks convenience features are available only with a home
    currency of US Dollars, such as the ability to download current exchange rates.
    Also, Intuit services such as payroll and online banking are only available in
    US Dollars. Once the home currency has been set and used in any transaction, it
    cannot be changed.
    """

    is_multi_currency_enabled: Optional[bool] = FieldInfo(alias="isMultiCurrencyEnabled", default=None)
    """Indicates whether the multicurrency feature is enabled for this company file.

    Once multicurrency is enabled for a company file, it cannot be disabled.
    """


class MultiLocationInventory(BaseModel):
    is_multi_location_inventory_available: Optional[bool] = FieldInfo(
        alias="isMultiLocationInventoryAvailable", default=None
    )
    """
    Indicates whether the multilocation inventory feature is available for this
    company file. When `true`, the feature can potentially be enabled.
    """

    is_multi_location_inventory_enabled: Optional[bool] = FieldInfo(
        alias="isMultiLocationInventoryEnabled", default=None
    )
    """
    Indicates whether the multilocation inventory feature is enabled for this
    company file. When `true`, inventory can be tracked across multiple locations.
    """


class PurchasesAndVendorsDefaultDiscountAccount(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class PurchasesAndVendors(BaseModel):
    days_bills_are_due: float = FieldInfo(alias="daysBillsAreDue")
    """
    The default number of days after receipt when bills are due for this company
    file.
    """

    default_discount_account: Optional[PurchasesAndVendorsDefaultDiscountAccount] = FieldInfo(
        alias="defaultDiscountAccount", default=None
    )
    """The account used to track vendor discounts for this company file."""

    is_automatically_using_discounts: bool = FieldInfo(alias="isAutomaticallyUsingDiscounts")
    """
    Indicates whether this company file is configured to automatically apply
    available vendor discounts or credits when paying bills.
    """

    is_using_inventory: bool = FieldInfo(alias="isUsingInventory")
    """Indicates whether this company file has inventory-related features enabled."""


class Reports(BaseModel):
    aging_report_basis: Literal["age_from_due_date", "age_from_transaction_date"] = FieldInfo(alias="agingReportBasis")
    """
    Determines how the aging periods are calculated in accounts receivable and
    accounts payable reports for this company file. When set to `age_from_due_date`,
    the overdue days shown in these reports begin with the due date on the invoice.
    When set to `age_from_transaction_date`, the overdue days begin with the date
    the transaction was created.
    """

    summary_report_basis: Literal["accrual", "cash"] = FieldInfo(alias="summaryReportBasis")
    """
    Indicates whether summary reports for this company file use cash-basis or
    accrual-basis bookkeeping. With `accrual` basis, transactions are recorded when
    they occur regardless of when payment is received or made. With `cash` basis,
    transactions are recorded only when payment is received or made.
    """


class SalesAndCustomersDefaultShippingMethod(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class SalesAndCustomersPriceLevels(BaseModel):
    is_rounding_sales_price_up: Optional[bool] = FieldInfo(alias="isRoundingSalesPriceUp", default=None)
    """
    Indicates whether this company file is configured to round amounts up to the
    nearest whole dollar for fixed percentage price levels. This setting does not
    affect per-item price levels.
    """

    is_using_price_levels: bool = FieldInfo(alias="isUsingPriceLevels")
    """Indicates whether this company file has price levels enabled.

    When `true`, price levels can be created and used to automatically calculate
    custom pricing for different customers.
    """


class SalesAndCustomers(BaseModel):
    default_markup_percentage: Optional[str] = FieldInfo(alias="defaultMarkupPercentage", default=None)
    """
    The default percentage that an inventory item will be marked up from its cost
    for this company file.
    """

    default_shipment_origin: Optional[str] = FieldInfo(alias="defaultShipmentOrigin", default=None)
    """
    The default shipment-origin location (i.e., FOB - freight on board) from which
    invoiced products are shipped for this company file. This indicates the point at
    which ownership and liability for goods transfer from seller to buyer.
    """

    default_shipping_method: Optional[SalesAndCustomersDefaultShippingMethod] = FieldInfo(
        alias="defaultShippingMethod", default=None
    )
    """
    The default shipping method used in all "Ship Via" fields for this company file.
    """

    is_auto_applying_payments: bool = FieldInfo(alias="isAutoApplyingPayments")
    """
    Indicates whether this company file is configured to automatically apply a
    customer's payment to their outstanding invoices, beginning with the oldest
    invoice.
    """

    is_tracking_reimbursed_expenses_as_income: bool = FieldInfo(alias="isTrackingReimbursedExpensesAsIncome")
    """
    Indicates whether this company file is configured to track an expense and the
    customer's reimbursement for that expense in separate accounts. When `true`,
    reimbursements can be tracked as income rather than as a reduction of the
    original expense.
    """

    price_levels: Optional[SalesAndCustomersPriceLevels] = FieldInfo(alias="priceLevels", default=None)
    """
    The custom pricing settings for this company file that can be assigned to
    specific customers. When a price level is set for a customer, QuickBooks
    automatically applies these custom prices to new invoices, sales receipts, sales
    orders, and credit memos. These settings can be overridden when creating
    individual transactions, and price levels can also be specified on individual
    line items in supported sales transactions.
    """


class SalesTaxDefaultItemSalesTax(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class SalesTaxDefaultNonTaxableSalesTaxCode(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class SalesTaxDefaultTaxableSalesTaxCode(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class SalesTax(BaseModel):
    default_item_sales_tax: SalesTaxDefaultItemSalesTax = FieldInfo(alias="defaultItemSalesTax")
    """The default tax code for sales for this company file."""

    default_non_taxable_sales_tax_code: SalesTaxDefaultNonTaxableSalesTaxCode = FieldInfo(
        alias="defaultNonTaxableSalesTaxCode"
    )
    """The default tax code for non-taxable sales for this company file."""

    default_taxable_sales_tax_code: SalesTaxDefaultTaxableSalesTaxCode = FieldInfo(alias="defaultTaxableSalesTaxCode")
    """The default tax code for taxable sales for this company file."""

    is_using_customer_tax_code: Optional[bool] = FieldInfo(alias="isUsingCustomerTaxCode", default=None)
    """
    Indicates whether this company file is configured to use tax codes for
    customers.
    """

    is_using_tax_inclusive_prices: Optional[bool] = FieldInfo(alias="isUsingTaxInclusivePrices", default=None)
    """
    Indicates whether this company file is configured to allow tax-inclusive prices.
    """

    is_using_vendor_tax_code: Optional[bool] = FieldInfo(alias="isUsingVendorTaxCode", default=None)
    """Indicates whether this company file is configured to use tax codes for vendors."""

    sales_tax_reporting_frequency: Literal["monthly", "quarterly", "annually"] = FieldInfo(
        alias="salesTaxReportingFrequency"
    )
    """The frequency at which sales tax reports are generated for this company file."""


class TimeTracking(BaseModel):
    first_day_of_week: Literal["monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"] = (
        FieldInfo(alias="firstDayOfWeek")
    )
    """The first day of a weekly timesheet period for this company file."""


class Preferences(BaseModel):
    accounting: Accounting
    """The accounting preferences for this company file."""

    app_access_rights: AppAccessRights = FieldInfo(alias="appAccessRights")
    """The current application access rights for this company file."""

    finance_charges: FinanceCharges = FieldInfo(alias="financeCharges")
    """The finance charge preferences for this company file.

    These settings determine how late payment charges are calculated and applied to
    customer accounts.
    """

    items_and_inventory: Optional[ItemsAndInventory] = FieldInfo(alias="itemsAndInventory", default=None)
    """The item inventory preferences for this company file."""

    jobs_and_estimates: JobsAndEstimates = FieldInfo(alias="jobsAndEstimates")
    """The jobs and estimates preferences for this company file."""

    multi_currency: Optional[MultiCurrency] = FieldInfo(alias="multiCurrency", default=None)
    """The multi-currency preferences for this company file."""

    multi_location_inventory: Optional[MultiLocationInventory] = FieldInfo(alias="multiLocationInventory", default=None)
    """The multi-location inventory preferences for this company file."""

    purchases_and_vendors: PurchasesAndVendors = FieldInfo(alias="purchasesAndVendors")
    """The purchases and vendors preferences for this company file."""

    reports: Reports
    """The reporting preferences for this company file."""

    sales_and_customers: SalesAndCustomers = FieldInfo(alias="salesAndCustomers")
    """The sales and customers preferences for this company file."""

    sales_tax: Optional[SalesTax] = FieldInfo(alias="salesTax", default=None)
    """The sales-tax preferences for this company file.

    If sales tax is turned off in the user interface (that is, if "No" is selected
    for "Do You Charge Sales Tax?" in the sales tax preferences), then this field
    will be `null`.
    """

    time_tracking: Optional[TimeTracking] = FieldInfo(alias="timeTracking", default=None)
    """The time-tracking preferences for this company file.

    If time tracking is turned off in the user interface (that is, if "No" is
    selected for "Do You Track Time?" in the time tracking preferences), then this
    field will be `null`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import date
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["InventoryAdjustmentListParams"]


class InventoryAdjustmentListParams(TypedDict, total=False):
    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    account_ids: Annotated[List[str], PropertyInfo(alias="accountIds")]
    """Filter for inventory adjustments associated with these accounts."""

    customer_ids: Annotated[List[str], PropertyInfo(alias="customerIds")]
    """Filter for inventory adjustments associated with these customers."""

    ids: List[str]
    """
    Filter for specific inventory adjustments by their QuickBooks-assigned unique
    identifier(s).

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    include_line_items: Annotated[bool, PropertyInfo(alias="includeLineItems")]
    """Whether to include line items in the response. Defaults to `true`."""

    item_ids: Annotated[List[str], PropertyInfo(alias="itemIds")]
    """Filter for inventory adjustments containing these inventory items."""

    limit: int
    """The maximum number of objects to return.

    **IMPORTANT**: QuickBooks Desktop does not support cursor-based pagination for
    inventory adjustments. This parameter will limit the response size, but you
    cannot fetch subsequent results using a cursor. For pagination, use the
    name-range parameters instead (e.g., `nameFrom=A&nameTo=B`).

    When this parameter is omitted, the endpoint returns all inventory adjustments
    without limit, unlike paginated endpoints which default to 150 records. This is
    acceptable because inventory adjustments typically have low record counts.
    """

    ref_number_contains: Annotated[str, PropertyInfo(alias="refNumberContains")]
    """Filter for inventory adjustments whose `refNumber` contains this substring.

    NOTE: If you use this parameter, you cannot also use `refNumberStartsWith` or
    `refNumberEndsWith`.
    """

    ref_number_ends_with: Annotated[str, PropertyInfo(alias="refNumberEndsWith")]
    """Filter for inventory adjustments whose `refNumber` ends with this substring.

    NOTE: If you use this parameter, you cannot also use `refNumberContains` or
    `refNumberStartsWith`.
    """

    ref_number_from: Annotated[str, PropertyInfo(alias="refNumberFrom")]
    """
    Filter for inventory adjustments whose `refNumber` is greater than or equal to
    this value. If omitted, the range will begin with the first number of the list.
    Uses a numerical comparison for values that contain only digits; otherwise, uses
    a lexicographical comparison.
    """

    ref_numbers: Annotated[List[str], PropertyInfo(alias="refNumbers")]
    """Filter for specific inventory adjustments by their ref-number(s),
    case-sensitive.

    In QuickBooks, ref-numbers are not required to be unique and can be arbitrarily
    changed by the QuickBooks user.

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    ref_number_starts_with: Annotated[str, PropertyInfo(alias="refNumberStartsWith")]
    """Filter for inventory adjustments whose `refNumber` starts with this substring.

    NOTE: If you use this parameter, you cannot also use `refNumberContains` or
    `refNumberEndsWith`.
    """

    ref_number_to: Annotated[str, PropertyInfo(alias="refNumberTo")]
    """
    Filter for inventory adjustments whose `refNumber` is less than or equal to this
    value. If omitted, the range will end with the last number of the list. Uses a
    numerical comparison for values that contain only digits; otherwise, uses a
    lexicographical comparison.
    """

    transaction_date_from: Annotated[Union[str, date], PropertyInfo(alias="transactionDateFrom", format="iso8601")]
    """
    Filter for inventory adjustments whose `date` field is on or after this date, in
    ISO 8601 format (YYYY-MM-DD).
    """

    transaction_date_to: Annotated[Union[str, date], PropertyInfo(alias="transactionDateTo", format="iso8601")]
    """
    Filter for inventory adjustments whose `date` field is on or before this date,
    in ISO 8601 format (YYYY-MM-DD).
    """

    updated_after: Annotated[str, PropertyInfo(alias="updatedAfter")]
    """
    Filter for inventory adjustments updated on or after this date and time, in ISO
    8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
    time is assumed to be 00:00:00 of that day.
    """

    updated_before: Annotated[str, PropertyInfo(alias="updatedBefore")]
    """
    Filter for inventory adjustments updated on or before this date and time, in ISO
    8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
    time is assumed to be 23:59:59 of that day.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["EstimateUpdateParams", "BillingAddress", "LineGroup", "LineGroupLine", "Line", "ShippingAddress"]


class EstimateUpdateParams(TypedDict, total=False):
    revision_number: Required[Annotated[str, PropertyInfo(alias="revisionNumber")]]
    """
    The current QuickBooks-assigned revision number of the estimate object you are
    updating, which you can get by fetching the object first. Provide the most
    recent `revisionNumber` to ensure you're working with the latest data;
    otherwise, the update will return an error.
    """

    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    billing_address: Annotated[BillingAddress, PropertyInfo(alias="billingAddress")]
    """The estimate's billing address."""

    class_id: Annotated[str, PropertyInfo(alias="classId")]
    """The estimate's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default. A class defined here is automatically used in this estimate's line
    items unless overridden at the line item level.
    """

    create_change_order: Annotated[bool, PropertyInfo(alias="createChangeOrder")]
    """
    When `true`, creates a "change order" that appears in this estimate's
    description field in QuickBooks's estimate form, specifying exactly what changed
    in this update request, the dollar amount of each change, and the net dollar
    change to this estimate.
    """

    customer_id: Annotated[str, PropertyInfo(alias="customerId")]
    """The customer or customer-job associated with this estimate."""

    customer_message_id: Annotated[str, PropertyInfo(alias="customerMessageId")]
    """The message to display to the customer on the estimate."""

    document_template_id: Annotated[str, PropertyInfo(alias="documentTemplateId")]
    """
    The predefined template in QuickBooks that determines the layout and formatting
    for this estimate when printed or displayed.
    """

    due_date: Annotated[Union[str, date], PropertyInfo(alias="dueDate", format="iso8601")]
    """The date by which this estimate must be paid, in ISO 8601 format (YYYY-MM-DD)."""

    exchange_rate: Annotated[float, PropertyInfo(alias="exchangeRate")]
    """
    The market exchange rate between this estimate's currency and the home currency
    in QuickBooks at the time of this transaction. Represented as a decimal value
    (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).
    """

    is_active: Annotated[bool, PropertyInfo(alias="isActive")]
    """Indicates whether this estimate is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """

    is_queued_for_email: Annotated[bool, PropertyInfo(alias="isQueuedForEmail")]
    """
    Indicates whether this estimate is included in the queue of documents for
    QuickBooks to email to the customer.
    """

    line_groups: Annotated[Iterable[LineGroup], PropertyInfo(alias="lineGroups")]
    """
    The estimate's line item groups, each representing a predefined set of related
    items.

    **IMPORTANT**:

    1. Including this array in your update request will **REPLACE** all existing
       line item groups for the estimate with this array. To keep any existing line
       item groups, you must include them in this array even if they have not
       changed. **Any line item groups not included will be removed.**

    2. To add a new line item group, include it here with the `id` field set to
       `-1`.

    3. If you do not wish to modify any line item groups, omit this field entirely
       to keep them unchanged.
    """

    lines: Iterable[Line]
    """The estimate's line items, each representing a single product or service quoted.

    **IMPORTANT**:

    1. Including this array in your update request will **REPLACE** all existing
       line items for the estimate with this array. To keep any existing line items,
       you must include them in this array even if they have not changed. **Any line
       items not included will be removed.**

    2. To add a new line item, include it here with the `id` field set to `-1`.

    3. If you do not wish to modify any line items, omit this field entirely to keep
       them unchanged.
    """

    memo: str
    """
    A memo or note for this estimate that appears in reports, but not on the
    estimate. Use `customerMessage` to add a note to this estimate.
    """

    other_custom_field: Annotated[str, PropertyInfo(alias="otherCustomField")]
    """A built-in custom field for additional information specific to this estimate.

    Unlike the user-defined fields in the `customFields` array, this is a standard
    QuickBooks field that exists for all estimates for convenience. Developers often
    use this field for tracking information that doesn't fit into other standard
    QuickBooks fields. Unlike `otherCustomField1` and `otherCustomField2`, which are
    line item fields, this exists at the transaction level. Hidden by default in the
    QuickBooks UI.
    """

    purchase_order_number: Annotated[str, PropertyInfo(alias="purchaseOrderNumber")]
    """The customer's Purchase Order (PO) number associated with this estimate.

    This field is often used to cross-reference the estimate with the customer's
    purchasing system.
    """

    ref_number: Annotated[str, PropertyInfo(alias="refNumber")]
    """
    The case-sensitive user-defined reference number for this estimate, which can be
    used to identify the transaction in QuickBooks. This value is not required to be
    unique and can be arbitrarily changed by the QuickBooks user.
    """

    sales_representative_id: Annotated[str, PropertyInfo(alias="salesRepresentativeId")]
    """The estimate's sales representative.

    Sales representatives can be employees, vendors, or other names in QuickBooks.
    """

    sales_tax_code_id: Annotated[str, PropertyInfo(alias="salesTaxCodeId")]
    """
    The sales-tax code for this estimate, determining whether it is taxable or
    non-taxable. This can be overridden at the transaction-line level.

    Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
    can also be created in QuickBooks. If QuickBooks is not set up to charge sales
    tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
    non-taxable code to all sales.
    """

    sales_tax_item_id: Annotated[str, PropertyInfo(alias="salesTaxItemId")]
    """
    The sales-tax item used to calculate the actual tax amount for this estimate's
    transactions by applying a specific tax rate collected for a single tax agency.
    Unlike `salesTaxCode`, which only indicates general taxability, this field
    drives the actual tax calculation and reporting.
    """

    shipment_origin: Annotated[str, PropertyInfo(alias="shipmentOrigin")]
    """
    The origin location from where the product associated with this estimate is
    shipped. This is the point at which ownership and liability for goods transfer
    from seller to buyer. Internally, QuickBooks uses the term "FOB" for this field,
    which stands for "freight on board". This field is informational and has no
    accounting implications.
    """

    shipping_address: Annotated[ShippingAddress, PropertyInfo(alias="shippingAddress")]
    """The estimate's shipping address."""

    terms_id: Annotated[str, PropertyInfo(alias="termsId")]
    """
    The estimate's payment terms, defining when payment is due and any applicable
    discounts.
    """

    transaction_date: Annotated[Union[str, date], PropertyInfo(alias="transactionDate", format="iso8601")]
    """The date of this estimate, in ISO 8601 format (YYYY-MM-DD)."""


class BillingAddress(TypedDict, total=False):
    city: str
    """The city, district, suburb, town, or village name of the address.

    Maximum length: 31 characters.
    """

    country: str
    """The country name of the address."""

    line1: str
    """The first line of the address (e.g., street, PO Box, or company name).

    Maximum length: 41 characters.
    """

    line2: str
    """
    The second line of the address, if needed (e.g., apartment, suite, unit, or
    building).

    Maximum length: 41 characters.
    """

    line3: str
    """The third line of the address, if needed.

    Maximum length: 41 characters.
    """

    line4: str
    """The fourth line of the address, if needed.

    Maximum length: 41 characters.
    """

    line5: str
    """The fifth line of the address, if needed.

    Maximum length: 41 characters.
    """

    note: str
    """
    A note written at the bottom of the address in the form in which it appears,
    such as the invoice form.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """The postal code or ZIP code of the address.

    Maximum length: 13 characters.
    """

    state: str
    """The state, county, province, or region name of the address.

    Maximum length: 21 characters.
    """


class LineGroupLine(TypedDict, total=False):
    id: Required[str]
    """
    The QuickBooks-assigned unique identifier of an existing estimate line you wish
    to retain or update.

    **IMPORTANT**: Set this field to `-1` for new estimate lines you wish to add.
    """

    amount: str
    """The monetary amount of this estimate line, represented as a decimal string.

    If both `quantity` and `rate` are specified but not `amount`, QuickBooks will
    use them to calculate `amount`. If `amount`, `rate`, and `quantity` are all
    unspecified, then QuickBooks will calculate `amount` based on a `quantity` of
    `1` and the suggested `rate`. This field cannot be cleared.
    """

    class_id: Annotated[str, PropertyInfo(alias="classId")]
    """The estimate line's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default. If a class is specified for the entire parent transaction, it is
    automatically applied to all estimate lines unless overridden here, at the
    transaction line level.
    """

    description: str
    """A description of this estimate line."""

    inventory_site_id: Annotated[str, PropertyInfo(alias="inventorySiteId")]
    """
    The site location where inventory for the item associated with this estimate
    line is stored.
    """

    inventory_site_location_id: Annotated[str, PropertyInfo(alias="inventorySiteLocationId")]
    """
    The specific location (e.g., bin or shelf) within the inventory site where the
    item associated with this estimate line is stored.
    """

    item_id: Annotated[str, PropertyInfo(alias="itemId")]
    """The item associated with this estimate line.

    This can refer to any good or service that the business buys or sells, including
    item types such as a service item, inventory item, or special calculation item
    like a discount item or sales-tax item.
    """

    markup_rate: Annotated[str, PropertyInfo(alias="markupRate")]
    """
    The markup that will be passed on to the customer for this item on this estimate
    line. `amount = (quantity * rate) * (1 + markupRate)`
    """

    markup_rate_percent: Annotated[str, PropertyInfo(alias="markupRatePercent")]
    """
    The markup, expressed as a percentage, that will be passed on to the customer
    for this item on this estimate line.
    `amount = (quantity * rate) * (1 + markupRatePercent/100)`
    """

    other_custom_field1: Annotated[str, PropertyInfo(alias="otherCustomField1")]
    """
    A built-in custom field for additional information specific to this estimate
    line. Unlike the user-defined fields in the `customFields` array, this is a
    standard QuickBooks field that exists for all estimate lines for convenience.
    Developers often use this field for tracking information that doesn't fit into
    other standard QuickBooks fields. Hidden by default in the QuickBooks UI.
    """

    other_custom_field2: Annotated[str, PropertyInfo(alias="otherCustomField2")]
    """
    A second built-in custom field for additional information specific to this
    estimate line. Unlike the user-defined fields in the `customFields` array, this
    is a standard QuickBooks field that exists for all estimate lines for
    convenience. Like `otherCustomField1`, developers often use this field for
    tracking information that doesn't fit into other standard QuickBooks fields.
    Hidden by default in the QuickBooks UI.
    """

    override_unit_of_measure_set_id: Annotated[str, PropertyInfo(alias="overrideUnitOfMeasureSetId")]
    """
    Specifies an alternative unit-of-measure set when updating this estimate line's
    `unitOfMeasure` field (e.g., "pound" or "kilogram"). This allows you to select
    units from a different set than the item's default unit-of-measure set, which
    remains unchanged on the item itself. The override applies only to this specific
    line. For example, you can sell an item typically measured in volume units using
    weight units in a specific transaction by specifying a different unit-of-measure
    set with this field.
    """

    price_level_id: Annotated[str, PropertyInfo(alias="priceLevelId")]
    """The price level applied to this estimate line.

    This overrides any price level set on the corresponding customer. The resulting
    estimate line will not show this price level, only the final `rate` calculated
    from it.
    """

    price_rule_conflict_strategy: Annotated[
        Literal["base_price", "zero"], PropertyInfo(alias="priceRuleConflictStrategy")
    ]
    """
    Specifies how to resolve price rule conflicts when adding or modifying this
    estimate line.
    """

    quantity: float
    """The quantity of the item associated with this estimate line.

    This field cannot be cleared.

    **NOTE**: Do not use this field if the associated item is a discount item.
    """

    rate: str
    """The price per unit for this estimate line.

    If both `rate` and `amount` are specified, `rate` will be ignored. If both
    `quantity` and `amount` are specified but not `rate`, QuickBooks will use them
    to calculate `rate`. Represented as a decimal string. This field cannot be
    cleared.
    """

    rate_percent: Annotated[str, PropertyInfo(alias="ratePercent")]
    """The price of this estimate line expressed as a percentage.

    Typically used for discount or markup items.
    """

    sales_tax_code_id: Annotated[str, PropertyInfo(alias="salesTaxCodeId")]
    """
    The sales-tax code for this estimate line, determining whether it is taxable or
    non-taxable. If set, this overrides any sales-tax codes defined on the parent
    transaction or the associated item.

    Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
    can also be created in QuickBooks. If QuickBooks is not set up to charge sales
    tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
    non-taxable code to all sales.
    """

    unit_of_measure: Annotated[str, PropertyInfo(alias="unitOfMeasure")]
    """The unit-of-measure used for the `quantity` in this estimate line.

    Must be a valid unit within the item's available units of measure.
    """


class LineGroup(TypedDict, total=False):
    id: Required[str]
    """
    The QuickBooks-assigned unique identifier of an existing estimate line group you
    wish to retain or update.

    **IMPORTANT**: Set this field to `-1` for new estimate line groups you wish to
    add.
    """

    item_group_id: Annotated[str, PropertyInfo(alias="itemGroupId")]
    """
    The estimate line group's item group, representing a predefined set of items
    bundled because they are commonly purchased together or grouped for faster
    entry.
    """

    lines: Iterable[LineGroupLine]
    """
    The estimate line group's line items, each representing a single product or
    service quoted.

    **IMPORTANT**:

    1. Including this array in your update request will **REPLACE** all existing
       line items for the estimate line group with this array. To keep any existing
       line items, you must include them in this array even if they have not
       changed. **Any line items not included will be removed.**

    2. To add a new line item, include it here with the `id` field set to `-1`.

    3. If you do not wish to modify any line items, omit this field entirely to keep
       them unchanged.
    """

    override_unit_of_measure_set_id: Annotated[str, PropertyInfo(alias="overrideUnitOfMeasureSetId")]
    """
    Specifies an alternative unit-of-measure set when updating this estimate line
    group's `unitOfMeasure` field (e.g., "pound" or "kilogram"). This allows you to
    select units from a different set than the item's default unit-of-measure set,
    which remains unchanged on the item itself. The override applies only to this
    specific line. For example, you can sell an item typically measured in volume
    units using weight units in a specific transaction by specifying a different
    unit-of-measure set with this field.
    """

    quantity: float
    """The quantity of the item group associated with this estimate line group.

    This field cannot be cleared.

    **NOTE**: Do not use this field if the associated item group is a discount item
    group.
    """

    unit_of_measure: Annotated[str, PropertyInfo(alias="unitOfMeasure")]
    """The unit-of-measure used for the `quantity` in this estimate line group.

    Must be a valid unit within the item's available units of measure.
    """


class Line(TypedDict, total=False):
    id: Required[str]
    """
    The QuickBooks-assigned unique identifier of an existing estimate line you wish
    to retain or update.

    **IMPORTANT**: Set this field to `-1` for new estimate lines you wish to add.
    """

    amount: str
    """The monetary amount of this estimate line, represented as a decimal string.

    If both `quantity` and `rate` are specified but not `amount`, QuickBooks will
    use them to calculate `amount`. If `amount`, `rate`, and `quantity` are all
    unspecified, then QuickBooks will calculate `amount` based on a `quantity` of
    `1` and the suggested `rate`. This field cannot be cleared.
    """

    class_id: Annotated[str, PropertyInfo(alias="classId")]
    """The estimate line's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default. If a class is specified for the entire parent transaction, it is
    automatically applied to all estimate lines unless overridden here, at the
    transaction line level.
    """

    description: str
    """A description of this estimate line."""

    inventory_site_id: Annotated[str, PropertyInfo(alias="inventorySiteId")]
    """
    The site location where inventory for the item associated with this estimate
    line is stored.
    """

    inventory_site_location_id: Annotated[str, PropertyInfo(alias="inventorySiteLocationId")]
    """
    The specific location (e.g., bin or shelf) within the inventory site where the
    item associated with this estimate line is stored.
    """

    item_id: Annotated[str, PropertyInfo(alias="itemId")]
    """The item associated with this estimate line.

    This can refer to any good or service that the business buys or sells, including
    item types such as a service item, inventory item, or special calculation item
    like a discount item or sales-tax item.
    """

    markup_rate: Annotated[str, PropertyInfo(alias="markupRate")]
    """
    The markup that will be passed on to the customer for this item on this estimate
    line. `amount = (quantity * rate) * (1 + markupRate)`
    """

    markup_rate_percent: Annotated[str, PropertyInfo(alias="markupRatePercent")]
    """
    The markup, expressed as a percentage, that will be passed on to the customer
    for this item on this estimate line.
    `amount = (quantity * rate) * (1 + markupRatePercent/100)`
    """

    other_custom_field1: Annotated[str, PropertyInfo(alias="otherCustomField1")]
    """
    A built-in custom field for additional information specific to this estimate
    line. Unlike the user-defined fields in the `customFields` array, this is a
    standard QuickBooks field that exists for all estimate lines for convenience.
    Developers often use this field for tracking information that doesn't fit into
    other standard QuickBooks fields. Hidden by default in the QuickBooks UI.
    """

    other_custom_field2: Annotated[str, PropertyInfo(alias="otherCustomField2")]
    """
    A second built-in custom field for additional information specific to this
    estimate line. Unlike the user-defined fields in the `customFields` array, this
    is a standard QuickBooks field that exists for all estimate lines for
    convenience. Like `otherCustomField1`, developers often use this field for
    tracking information that doesn't fit into other standard QuickBooks fields.
    Hidden by default in the QuickBooks UI.
    """

    override_unit_of_measure_set_id: Annotated[str, PropertyInfo(alias="overrideUnitOfMeasureSetId")]
    """
    Specifies an alternative unit-of-measure set when updating this estimate line's
    `unitOfMeasure` field (e.g., "pound" or "kilogram"). This allows you to select
    units from a different set than the item's default unit-of-measure set, which
    remains unchanged on the item itself. The override applies only to this specific
    line. For example, you can sell an item typically measured in volume units using
    weight units in a specific transaction by specifying a different unit-of-measure
    set with this field.
    """

    price_level_id: Annotated[str, PropertyInfo(alias="priceLevelId")]
    """The price level applied to this estimate line.

    This overrides any price level set on the corresponding customer. The resulting
    estimate line will not show this price level, only the final `rate` calculated
    from it.
    """

    price_rule_conflict_strategy: Annotated[
        Literal["base_price", "zero"], PropertyInfo(alias="priceRuleConflictStrategy")
    ]
    """
    Specifies how to resolve price rule conflicts when adding or modifying this
    estimate line.
    """

    quantity: float
    """The quantity of the item associated with this estimate line.

    This field cannot be cleared.

    **NOTE**: Do not use this field if the associated item is a discount item.
    """

    rate: str
    """The price per unit for this estimate line.

    If both `rate` and `amount` are specified, `rate` will be ignored. If both
    `quantity` and `amount` are specified but not `rate`, QuickBooks will use them
    to calculate `rate`. Represented as a decimal string. This field cannot be
    cleared.
    """

    rate_percent: Annotated[str, PropertyInfo(alias="ratePercent")]
    """The price of this estimate line expressed as a percentage.

    Typically used for discount or markup items.
    """

    sales_tax_code_id: Annotated[str, PropertyInfo(alias="salesTaxCodeId")]
    """
    The sales-tax code for this estimate line, determining whether it is taxable or
    non-taxable. If set, this overrides any sales-tax codes defined on the parent
    transaction or the associated item.

    Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
    can also be created in QuickBooks. If QuickBooks is not set up to charge sales
    tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
    non-taxable code to all sales.
    """

    unit_of_measure: Annotated[str, PropertyInfo(alias="unitOfMeasure")]
    """The unit-of-measure used for the `quantity` in this estimate line.

    Must be a valid unit within the item's available units of measure.
    """


class ShippingAddress(TypedDict, total=False):
    city: str
    """The city, district, suburb, town, or village name of the address.

    Maximum length: 31 characters.
    """

    country: str
    """The country name of the address."""

    line1: str
    """The first line of the address (e.g., street, PO Box, or company name).

    Maximum length: 41 characters.
    """

    line2: str
    """
    The second line of the address, if needed (e.g., apartment, suite, unit, or
    building).

    Maximum length: 41 characters.
    """

    line3: str
    """The third line of the address, if needed.

    Maximum length: 41 characters.
    """

    line4: str
    """The fourth line of the address, if needed.

    Maximum length: 41 characters.
    """

    line5: str
    """The fifth line of the address, if needed.

    Maximum length: 41 characters.
    """

    note: str
    """
    A note written at the bottom of the address in the form in which it appears,
    such as the invoice form.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """The postal code or ZIP code of the address.

    Maximum length: 13 characters.
    """

    state: str
    """The state, county, province, or region name of the address.

    Maximum length: 21 characters.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PayrollWageItemCreateParams"]


class PayrollWageItemCreateParams(TypedDict, total=False):
    expense_account_id: Required[Annotated[str, PropertyInfo(alias="expenseAccountId")]]
    """
    The expense account used to track wage expenses paid through this payroll wage
    item.
    """

    name: Required[str]
    """
    The case-insensitive unique name of this payroll wage item, unique across all
    payroll wage items.

    **NOTE**: Payroll wage items do not have a `fullName` field because they are not
    hierarchical objects, which is why `name` is unique for them but not for objects
    that have parents.

    Maximum length: 31 characters.
    """

    wage_type: Required[
        Annotated[
            Literal[
                "bonus",
                "commission",
                "hourly_overtime",
                "hourly_regular",
                "hourly_sick",
                "hourly_vacation",
                "salary_regular",
                "salary_sick",
                "salary_vacation",
            ],
            PropertyInfo(alias="wageType"),
        ]
    ]
    """
    Categorizes how this payroll wage item calculates pay - can be hourly (regular,
    overtime, sick, or vacation), salary (regular, sick, or vacation), bonus, or
    commission based.
    """

    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    is_active: Annotated[bool, PropertyInfo(alias="isActive")]
    """Indicates whether this payroll wage item is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ServiceItemUpdateParams", "Barcode", "SalesAndPurchaseDetails", "SalesOrPurchaseDetails"]


class ServiceItemUpdateParams(TypedDict, total=False):
    revision_number: Required[Annotated[str, PropertyInfo(alias="revisionNumber")]]
    """
    The current QuickBooks-assigned revision number of the service item object you
    are updating, which you can get by fetching the object first. Provide the most
    recent `revisionNumber` to ensure you're working with the latest data;
    otherwise, the update will return an error.
    """

    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    barcode: Barcode
    """The service item's barcode."""

    class_id: Annotated[str, PropertyInfo(alias="classId")]
    """The service item's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default.
    """

    force_unit_of_measure_change: Annotated[bool, PropertyInfo(alias="forceUnitOfMeasureChange")]
    """
    Indicates whether to allow changing the service item's unit-of-measure set
    (using the `unitOfMeasureSetId` field) when the base unit of the new
    unit-of-measure set does not match that of the currently assigned set. Without
    setting this field to `true` in this scenario, the request will fail with an
    error; hence, this field is equivalent to accepting the warning prompt in the
    QuickBooks UI.

    NOTE: Changing the base unit requires you to update the item's
    quantities-on-hand and cost to reflect the new unit; otherwise, these values
    will be inaccurate. Alternatively, consider creating a new item with the desired
    unit-of-measure set and deactivating the old item.
    """

    is_active: Annotated[bool, PropertyInfo(alias="isActive")]
    """Indicates whether this service item is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """

    name: str
    """The case-insensitive name of this service item.

    Not guaranteed to be unique because it does not include the names of its
    hierarchical parent objects like `fullName` does. For example, two service items
    could both have the `name` "Web-Design", but they could have unique `fullName`
    values, such as "Consulting:Web-Design" and "Contracting:Web-Design".

    Maximum length: 31 characters.
    """

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """The parent service item one level above this one in the hierarchy.

    For example, if this service item has a `fullName` of "Consulting:Web-Design",
    its parent has a `fullName` of "Consulting". If this service item is at the top
    level, this field will be `null`.
    """

    sales_and_purchase_details: Annotated[SalesAndPurchaseDetails, PropertyInfo(alias="salesAndPurchaseDetails")]
    """
    Details for service items that are both purchased and sold, such as reimbursable
    expenses or inventory items that are bought from vendors and sold to customers.

    **IMPORTANT**: You cannot specify both `salesAndPurchaseDetails` and
    `salesOrPurchaseDetails` when modifying a service item because an item cannot
    have both configurations.
    """

    sales_or_purchase_details: Annotated[SalesOrPurchaseDetails, PropertyInfo(alias="salesOrPurchaseDetails")]
    """
    Details for service items that are exclusively sold or exclusively purchased,
    but not both. This typically applies to non-inventory items (like a purchased
    office supply that isn't resold) or service items (like consulting services that
    are sold but not purchased).

    **IMPORTANT**: You cannot specify both `salesOrPurchaseDetails` and
    `salesAndPurchaseDetails` when modifying a service item because an item cannot
    have both configurations.
    """

    sales_tax_code_id: Annotated[str, PropertyInfo(alias="salesTaxCodeId")]
    """
    The default sales-tax code for this service item, determining whether it is
    taxable or non-taxable. This can be overridden at the transaction-line level.

    Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
    can also be created in QuickBooks. If QuickBooks is not set up to charge sales
    tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
    non-taxable code to all sales.
    """

    unit_of_measure_set_id: Annotated[str, PropertyInfo(alias="unitOfMeasureSetId")]
    """
    The unit-of-measure set associated with this service item, which consists of a
    base unit and related units.
    """


class Barcode(TypedDict, total=False):
    allow_override: Annotated[bool, PropertyInfo(alias="allowOverride")]
    """Indicates whether to allow the barcode to be overridden."""

    assign_even_if_used: Annotated[bool, PropertyInfo(alias="assignEvenIfUsed")]
    """Indicates whether to assign the barcode even if it is already used."""

    value: str
    """The item's barcode value."""


class SalesAndPurchaseDetails(TypedDict, total=False):
    expense_account_id: Annotated[str, PropertyInfo(alias="expenseAccountId")]
    """The expense account used to track costs from purchases of this item."""

    income_account_id: Annotated[str, PropertyInfo(alias="incomeAccountId")]
    """The income account used to track revenue from sales of this item."""

    preferred_vendor_id: Annotated[str, PropertyInfo(alias="preferredVendorId")]
    """The preferred vendor from whom this item is typically purchased."""

    purchase_cost: Annotated[str, PropertyInfo(alias="purchaseCost")]
    """
    The cost at which this item is purchased from vendors, represented as a decimal
    string.
    """

    purchase_description: Annotated[str, PropertyInfo(alias="purchaseDescription")]
    """
    The description of this item that appears on purchase forms (e.g., checks,
    bills, item receipts) when it is ordered or bought from vendors.
    """

    purchase_tax_code_id: Annotated[str, PropertyInfo(alias="purchaseTaxCodeId")]
    """The tax code applied to purchases of this item.

    Applicable in regions where purchase taxes are used, such as Canada or the UK.
    """

    sales_description: Annotated[str, PropertyInfo(alias="salesDescription")]
    """
    The description of this item that appears on sales forms (e.g., invoices, sales
    receipts) when sold to customers.
    """

    sales_price: Annotated[str, PropertyInfo(alias="salesPrice")]
    """
    The price at which this item is sold to customers, represented as a decimal
    string.
    """

    update_existing_transactions_expense_account: Annotated[
        bool, PropertyInfo(alias="updateExistingTransactionsExpenseAccount")
    ]
    """
    When `true`, applies the new expense account (specified by the
    `expenseAccountId` field) to all existing transactions that use this item. This
    updates historical data and should be used with caution. The update will fail if
    any affected transaction falls within a closed accounting period. If this
    parameter is not specified, QuickBooks will prompt the user before making any
    changes.
    """

    update_existing_transactions_income_account: Annotated[
        bool, PropertyInfo(alias="updateExistingTransactionsIncomeAccount")
    ]
    """
    When `true`, applies the new income account (specified by the `incomeAccountId`
    field) to all existing transactions that use this item. This updates historical
    data and should be used with caution. The update will fail if any affected
    transaction falls within a closed accounting period. If this parameter is not
    specified, QuickBooks will prompt the user before making any changes.
    """


class SalesOrPurchaseDetails(TypedDict, total=False):
    description: str
    """A description of this item."""

    posting_account_id: Annotated[str, PropertyInfo(alias="postingAccountId")]
    """The posting account to which transactions involving this item are posted.

    This could be an income account when selling or an expense account when
    purchasing.
    """

    price: str
    """
    The price at which this item is purchased or sold, represented as a decimal
    string.
    """

    price_percentage: Annotated[str, PropertyInfo(alias="pricePercentage")]
    """
    The price of this item expressed as a percentage, used instead of `price` when
    the item's cost is calculated as a percentage of another amount. For example, a
    service item that costs a percentage of another item's price.
    """

    update_existing_transactions_account: Annotated[bool, PropertyInfo(alias="updateExistingTransactionsAccount")]
    """
    When `true`, applies the new account (specified by the `accountId` field) to all
    existing transactions associated with this item. This updates historical data
    and should be used with caution. The update will fail if any affected
    transaction falls within a closed accounting period. If this parameter is not
    specified, QuickBooks will prompt the user before making any changes.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "ReceivePaymentCreateParams",
    "ApplyToTransaction",
    "ApplyToTransactionApplyCredit",
    "CreditCardTransaction",
    "CreditCardTransactionRequest",
    "CreditCardTransactionResponse",
]


class ReceivePaymentCreateParams(TypedDict, total=False):
    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """
    The customer or customer-job to which the payment for this receive-payment is
    credited.
    """

    total_amount: Required[Annotated[str, PropertyInfo(alias="totalAmount")]]
    """
    The total monetary amount of this receive-payment, represented as a decimal
    string.

    **NOTE**: The sum of the `paymentAmount` amounts in the `applyToTransactions`
    array cannot exceed the `totalAmount`, or you will receive an error.
    """

    transaction_date: Required[Annotated[Union[str, date], PropertyInfo(alias="transactionDate", format="iso8601")]]
    """The date of this receive-payment, in ISO 8601 format (YYYY-MM-DD)."""

    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    apply_to_transactions: Annotated[Iterable[ApplyToTransaction], PropertyInfo(alias="applyToTransactions")]
    """The invoices to be paid by this receive-payment.

    This will create a link between this receive-payment and the specified invoices.

    **IMPORTANT**: In each `applyToTransactions` object, you must specify either
    `paymentAmount`, `applyCredits`, `discountAmount`, or any combination of these;
    if none of these are specified, you will receive an error for an empty
    transaction.

    **IMPORTANT**: The target invoice must have `isPaid=false`, otherwise,
    QuickBooks will report this object as "cannot be found".

    **NOTE**: You must specify either `isAutoApply` or `applyToTransactions` when
    creating a receive-payment, but never both.
    """

    credit_card_transaction: Annotated[CreditCardTransaction, PropertyInfo(alias="creditCardTransaction")]
    """
    The credit card transaction data for this receive-payment's payment when using
    QuickBooks Merchant Services (QBMS). If specifying this field, you must also
    specify the `paymentMethod` field.
    """

    deposit_to_account_id: Annotated[str, PropertyInfo(alias="depositToAccountId")]
    """
    The account where the funds for this receive-payment will be or have been
    deposited. If omitted, QuickBooks will use the default Undeposited Funds
    account.
    """

    exchange_rate: Annotated[float, PropertyInfo(alias="exchangeRate")]
    """
    The market exchange rate between this receive-payment's currency and the home
    currency in QuickBooks at the time of this transaction. Represented as a decimal
    value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).
    """

    external_id: Annotated[str, PropertyInfo(alias="externalId")]
    """
    A globally unique identifier (GUID) you, the developer, can provide for tracking
    this object in your external system. This field is immutable and can only be set
    during object creation.

    **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
    QuickBooks will return an error.
    """

    is_auto_apply: Annotated[bool, PropertyInfo(alias="isAutoApply")]
    """
    When `true`, QuickBooks applies `totalAmount` to any outstanding transaction
    that exactly matches `totalAmount`. If no exact match is found, this
    receive-payment is applied to the oldest outstanding transaction for the
    customer-job. When `false`, QuickBooks records the payment but does not apply it
    to any specific transaction, causing the amount to appear as a credit on the
    customer-job's next transaction.

    **IMPORTANT**: You must specify either `isAutoApply` or `applyToTransactions`
    when creating a receive-payment, but never both.
    """

    memo: str
    """
    A memo or note for this receive-payment that will be displayed at the beginning
    of reports containing details about this receive-payment.
    """

    payment_method_id: Annotated[str, PropertyInfo(alias="paymentMethodId")]
    """The receive-payment's payment method (e.g., cash, check, credit card).

    **NOTE**: If this receive-payment contains credit card transaction data supplied
    from QuickBooks Merchant Services (QBMS) transaction responses, you must specify
    a credit card payment method (e.g., "Visa", "MasterCard", etc.).
    """

    receivables_account_id: Annotated[str, PropertyInfo(alias="receivablesAccountId")]
    """
    The Accounts-Receivable (A/R) account to which this receive-payment is assigned,
    used to track the amount owed. If not specified, QuickBooks Desktop will use its
    default A/R account.

    **IMPORTANT**: If this receive-payment is linked to other transactions, this A/R
    account must match the `receivablesAccount` used in all linked transactions. For
    example, when refunding a credit card payment, the A/R account must match the
    one used in the original credit transactions being refunded.
    """

    ref_number: Annotated[str, PropertyInfo(alias="refNumber")]
    """
    The case-sensitive user-defined reference number for this receive-payment, which
    can be used to identify the transaction in QuickBooks. This value is not
    required to be unique and can be arbitrarily changed by the QuickBooks user.
    When left blank in this create request, this field will be left blank in
    QuickBooks (i.e., it does _not_ auto-increment).
    """


class ApplyToTransactionApplyCredit(TypedDict, total=False):
    applied_amount: Required[Annotated[str, PropertyInfo(alias="appliedAmount")]]
    """The amount of credit applied to this transaction.

    This could include customer deposits, payments, or credits. Represented as a
    decimal string.
    """

    credit_transaction_id: Required[Annotated[str, PropertyInfo(alias="creditTransactionId")]]
    """
    The unique identifier of the credit transaction (credit memo or vendor credit)
    to apply to this transaction.
    """

    override_credit_application: Annotated[bool, PropertyInfo(alias="overrideCreditApplication")]
    """Indicates whether to override the credit."""


class ApplyToTransaction(TypedDict, total=False):
    transaction_id: Required[Annotated[str, PropertyInfo(alias="transactionId")]]
    """The ID of the receivable transaction to which this payment is applied."""

    apply_credits: Annotated[Iterable[ApplyToTransactionApplyCredit], PropertyInfo(alias="applyCredits")]
    """Credit memos to apply to this receivable transaction, reducing its balance.

    This creates a link between this receivable transaction and the specified credit
    memos.

    **IMPORTANT**: By default, QuickBooks will not return any information about the
    linked transactions in this endpoint's response even when this request is
    successful. To see the transactions linked via this field, refetch the
    receivable transaction and check the `linkedTransactions` response field. If
    fetching a list of receivable transactions, you must also specify the parameter
    `includeLinkedTransactions=true` to see the `linkedTransactions` response field.
    """

    discount_account_id: Annotated[str, PropertyInfo(alias="discountAccountId")]
    """The financial account used to track this receivable transaction's discount."""

    discount_amount: Annotated[str, PropertyInfo(alias="discountAmount")]
    """
    The monetary amount by which to reduce the receivable transaction's receivable
    amount, represented as a decimal string.
    """

    discount_class_id: Annotated[str, PropertyInfo(alias="discountClassId")]
    """The class used to track this receivable transaction's discount."""

    payment_amount: Annotated[str, PropertyInfo(alias="paymentAmount")]
    """
    The monetary amount to apply to the receivable transaction, represented as a
    decimal string.
    """


class CreditCardTransactionRequest(TypedDict, total=False):
    expiration_month: Required[Annotated[float, PropertyInfo(alias="expirationMonth")]]
    """The month when the credit card expires."""

    expiration_year: Required[Annotated[float, PropertyInfo(alias="expirationYear")]]
    """The year when the credit card expires."""

    name: Required[str]
    """The cardholder's name on the card."""

    number: Required[str]
    """The credit card number. Must be masked with lower case "x" and no dashes."""

    address: str
    """The card's billing address."""

    commercial_card_code: Annotated[str, PropertyInfo(alias="commercialCardCode")]
    """
    The commercial card code identifies the type of business credit card being used
    (purchase, corporate, or business) for Visa and Mastercard transactions only.
    When provided, this code may qualify the transaction for lower processing fees
    compared to the standard rates that apply when no code is specified.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """The card's billing address ZIP or postal code."""

    transaction_mode: Annotated[Literal["card_not_present", "card_present"], PropertyInfo(alias="transactionMode")]
    """
    Indicates whether this credit card transaction came from a card swipe
    (`card_present`) or not (`card_not_present`).
    """

    transaction_type: Annotated[
        Literal["authorization", "capture", "charge", "refund", "voice_authorization"],
        PropertyInfo(alias="transactionType"),
    ]
    """The QBMS transaction type from which the current transaction data originated."""


class CreditCardTransactionResponse(TypedDict, total=False):
    credit_card_transaction_id: Required[Annotated[str, PropertyInfo(alias="creditCardTransactionId")]]
    """
    The ID returned from the credit card processor for this credit card transaction.
    """

    merchant_account_number: Required[Annotated[str, PropertyInfo(alias="merchantAccountNumber")]]
    """
    The QBMS account number of the merchant who is running this transaction using
    the customer's credit card.
    """

    payment_status: Required[Annotated[Literal["completed", "unknown"], PropertyInfo(alias="paymentStatus")]]
    """
    Indicates whether this credit card transaction is known to have been
    successfully processed by the card issuer.
    """

    status_code: Required[Annotated[float, PropertyInfo(alias="statusCode")]]
    """
    The status code returned in the original QBMS transaction response for this
    credit card transaction.
    """

    status_message: Required[Annotated[str, PropertyInfo(alias="statusMessage")]]
    """
    The status message returned in the original QBMS transaction response for this
    credit card transaction.
    """

    transaction_authorized_at: Required[Annotated[str, PropertyInfo(alias="transactionAuthorizedAt")]]
    """
    The date and time when the credit card processor authorized this credit card
    transaction.
    """

    authorization_code: Annotated[str, PropertyInfo(alias="authorizationCode")]
    """
    The authorization code returned from the credit card processor to indicate that
    this charge will be paid by the card issuer.
    """

    avs_street_status: Annotated[Literal["fail", "not_available", "pass"], PropertyInfo(alias="avsStreetStatus")]
    """
    Indicates whether the street address supplied in the transaction request matches
    the customer's address on file at the card issuer.
    """

    avs_zip_status: Annotated[Literal["fail", "not_available", "pass"], PropertyInfo(alias="avsZipStatus")]
    """
    Indicates whether the customer postal ZIP code supplied in the transaction
    request matches the customer's postal code recognized at the card issuer.
    """

    card_security_code_match: Annotated[
        Literal["fail", "not_available", "pass"], PropertyInfo(alias="cardSecurityCodeMatch")
    ]
    """
    Indicates whether the card security code supplied in the transaction request
    matches the card security code recognized for that credit card number at the
    card issuer.
    """

    client_transaction_id: Annotated[str, PropertyInfo(alias="clientTransactionId")]
    """
    A value returned from QBMS transactions for future use by the QuickBooks
    Reconciliation feature.
    """

    payment_grouping_code: Annotated[float, PropertyInfo(alias="paymentGroupingCode")]
    """
    An internal code returned by QuickBooks Merchant Services (QBMS) from the
    transaction request, needed for the QuickBooks reconciliation feature.
    """

    recon_batch_id: Annotated[str, PropertyInfo(alias="reconBatchId")]
    """
    An internal ID returned by QuickBooks Merchant Services (QBMS) from the
    transaction request, needed for the QuickBooks reconciliation feature.
    """

    transaction_authorization_stamp: Annotated[float, PropertyInfo(alias="transactionAuthorizationStamp")]
    """
    An internal value for this credit card transaction, needed for the QuickBooks
    reconciliation feature.
    """


class CreditCardTransaction(TypedDict, total=False):
    request: CreditCardTransactionRequest
    """
    The transaction request data originally supplied for this credit card
    transaction when using QuickBooks Merchant Services (QBMS).
    """

    response: CreditCardTransactionResponse
    """
    The transaction response data for this credit card transaction when using
    QuickBooks Merchant Services (QBMS).
    """
